<?php
/**
 * Library of functions and constants for module flashchat
 * This file should have two well differenced parts:
 *   - All the core Moodle functions, neeeded to allow
 *     the module to work integrated in Moodle.
 *   - All the flashchat specific functions, needed
 *     to implement all the module logic. Please, note
 *     that, if the module become complex and this lib
 *     grows a lot, it's HIGHLY recommended to move all
 *     these module specific functions to a new php file,
 *     called "locallib.php" (see forum, quiz...). This will
 *     help to save some memory when Moodle is performing
 *     actions across all modules.
 */



/**
 * Given an object containing all the necessary data, 
 * (defined by the form in mod.html) this function 
 * will create a new instance and return the id number 
 * of the new instance.
 *
 * @param object $instance An object from the form in mod.html
 * @return int The id of the newly inserted flashchat record
 **/
//require_once("../../config.php");
require_once ($CFG->dirroot.'/mod/flashchat/functions_chat.php');
$flashchat_CONSTANT = 7;
function flashchat_add_instance($flashchat) {
	global $DB;
    // temp added for debugging
    //echo "ADD INSTANCE CALLED";
   // print_object($flashchat);
	$data->name = $flashchat->name;
	$data->timecreated = time();
	$data->timemodified = time();
	$data->extendserver = $flashchat->extendserver;
	$data->servertype = !empty($flashchat->servertype) ? 1 : 0;
	$data->client_loc = $flashchat->client_loc ? ($flashchat->client_loc . (substr($flashchat->client_loc, -1,1) != '/' ? '/' : '')) : '';
	
	
	if($flashchat->extendserver == 2){
		$data->room = !empty($flashchat->room) ? $flashchat->room : 'Lobby';
		preg_match("/\w+$/", $data->room,$matches);
		if(strlen($data->room) != strlen($matches[0])){
			return false;
		}
	}elseif($flashchat->extendserver == 1){
		if(!empty($data->client_loc)){
			$serverinfo = get_server_host_port_host($data->client_loc);
			if($serverinfo){
				$data->server_host = empty($serverinfo['fc_server_host']) ? parse_url($data->client_loc,PHP_URL_HOST) : $serverinfo['fc_server_host'];
				$data->server_host_s = empty($serverinfo['fc_server_host_s']) ? '' : $serverinfo['fc_server_host_s'];
				$data->server_host_h = empty($serverinfo['fc_server_host_h']) ? '' : $serverinfo['fc_server_host_h'];
				
				$data->server_port =  empty($serverinfo['fc_server_port']) ? '' : $serverinfo['fc_server_port'];
				$data->server_port_h = empty($serverinfo['fc_server_port_h']) ? '' : $serverinfo['fc_server_port_h'];
            	$data->server_port_s = empty($serverinfo['fc_server_port_s']) ? '' : $serverinfo['fc_server_port_s'];
            	$data->api_url = 'http://' . $data->server_host . '/';
            	$data->group_name = substr(parse_url($data->client_loc, PHP_URL_PATH),1,-1);
				$c_own_loc = validate_client($data->client_loc);
				$da = validate_data_api($data->api_url . 'online.js?group=' . $data->group_name);
		        if($c_own_loc || $da){
		        	return false;
		        }
			}else{
				return false;
			}
		}else{
			return false;
		}
	}else{
		if($data->client_loc){
			$serverinfo = get_server_host_port_local($data->client_loc);
			$data->server_host = !empty($flashchat->server_host) ? $flashchat->server_host : (empty($serverinfo['fc_server_host']) ? $_SERVER['HTTP_HOST'] : $serverinfo['fc_server_host']) ;
            $data->server_host_s = empty($serverinfo['fc_server_host_s']) ? $data->server_host : $serverinfo['fc_server_host_s'];
            $data->server_host_h = empty($serverinfo['fc_server_host_h']) ? $data->server_host : $serverinfo['fc_server_host_h'];
            if(!is_numeric($flashchat->server_port) || strpos($flashchat->server_port ,".")!==false){
            	$data->server_port = empty($serverinfo['fc_server_port']) ? ( $data->servertype == 1 ? '51212' : '51127') : $serverinfo['fc_server_port'] ;
            }else{
            	$data->server_port = $flashchat->server_port;
            }
            if(!is_numeric($flashchat->server_port_h) || strpos($flashchat->server_port_h ,".")!==false){
            	$data->server_port_h = empty($serverinfo['fc_server_port_h']) ? ( $data->servertype == 1 ? '31212' : '35555') : $serverinfo['fc_server_port_h'] ;
            }else{
            	$data->server_port_h = $flashchat->server_port_h;
            }
            $data->server_port_s = empty($serverinfo['fc_server_port_s']) ? ( $data->servertype == 1 ? '51213' : '51128') : $serverinfo['fc_server_port_s'] ;
            $data->api_url = 'http://' . $data->server_host . ':' . $data->server_port_h . '/';
		}else{
			$data->server_host = $_SERVER['HTTP_HOST'];
            $data->server_host_h = $data->server_host_s = $data->server_host;
            if($data->servertype == 1){
            	$data->server_port_h = '31212';
            	$data->server_port_s = '51213';
            	$data->server_port = '51212';
            }else{
            	$data->server_port_h = '35555';
            	$data->server_port_s = '51128';
            	$data->server_port = '51127';
            }
            $data->api_url = 'http://' . $data->server_host . ':' . $data->server_port_h . '/';
            $data->client_loc = $data->api_url;
		}
		$s_own_loc = 1;
		$c_own_loc = 1;
		$da = 1;
		if($flashchat->server_loc){
			$c_own_loc = validate_client($data->client_loc);
			$da = validate_data_api($data->api_url . 'online.js');
			if($c_own_loc || $da){
 				return false;
			}
		}else{
			$c_own_loc = validate_client($data->client_loc);
			if($c_own_loc){
				return false;
			}
		}
		$check = validate_client('http://'.parse_url($data->client_loc, PHP_URL_HOST).':'.$data->server_port_h.'/');
		if($check){
			return false;
		}
	}	
	if($data->servertype == 1){
		$data->clienttype = 0;
	}else{
		$data->clienttype = !empty($flashchat->clienttype) ? 1 : 0;
	}	
	$data->fullscreen = !empty($flashchat->fullscreen) ? 1 : 0;	
	$data->room_list = !empty($flashchat->room_list) ? $flashchat->room_list : 0;
	$data->user_list = !empty($flashchat->user_list) ? $flashchat->user_list : 0;	
	$data->client_width = (!empty($flashchat->client_width) && is_numeric($flashchat->client_width)) ? $flashchat->client_width : 700;
	$data->client_height = (!empty($flashchat->client_height) && is_numeric($flashchat->client_height)) ? $flashchat->client_height : 642;
	$data->client_lang = !empty($flashchat->client_lang)? $flashchat->client_lang : 'auto';
	$data->client_skin = !empty($flashchat->client_skin)? $flashchat->client_skin : 'default';
	//var_dump($data);
	//exit;
    return $DB->insert_record("flashchat", $data);
}

/**
 * Given an object containing all the necessary data, 
 * (defined by the form in mod.html) this function 
 * will update an existing instance with new data.
 *
 * @param object $instance An object from the form in mod.html
 * @return boolean Success/Fail
 **/
function flashchat_update_instance($flashchat) {

	global $DB;
	$data->id = $flashchat->instance;
	//$data->name = $flashchat->name;
	$data->extendserver = $flashchat->extendserver;
	$data->client_loc = $flashchat->client_loc ? ($flashchat->client_loc . (substr($flashchat->client_loc, -1,1) != '/' ? '/' : '')) : '';
	$data->servertype = !empty($flashchat->servertype) ? 1 : 0;
	
	if($flashchat->extendserver == 2){
		$data->room = !empty($flashchat->room) ? $flashchat->room : 'Lobby';
		preg_match("/\w+$/", $data->room,$matches);
		if(strlen($data->room) != strlen($matches[0])){
			return false;
		}		
		$data->client_lang = !empty($flashchat->client_lang)? $flashchat->client_lang : 'auto';
		$data->client_skin = !empty($flashchat->client_skin)? $flashchat->client_skin : 'default';
	}elseif($flashchat->extendserver == 1){
		if(!empty($data->client_loc)){
			$serverinfo = get_server_host_port_host($data->client_loc);
			if($serverinfo){
				$data->server_host = empty($serverinfo['fc_server_host']) ? parse_url($data->client_loc,PHP_URL_HOST) : $serverinfo['fc_server_host'];
				$data->server_host_s = empty($serverinfo['fc_server_host_s']) ? '' : $serverinfo['fc_server_host_s'];
				$data->server_host_h = empty($serverinfo['fc_server_host_h']) ? '' : $serverinfo['fc_server_host_h'];
				
				$data->server_port =  empty($serverinfo['fc_server_port']) ? '' : $serverinfo['fc_server_port'];
				$data->server_port_h = empty($serverinfo['fc_server_port_h']) ? '' : $serverinfo['fc_server_port_h'];
            	$data->server_port_s = empty($serverinfo['fc_server_port_s']) ? '' : $serverinfo['fc_server_port_s'];
            	$data->api_url = 'http://' . $data->server_host . '/';
            	$data->group_name = substr(parse_url($data->client_loc, PHP_URL_PATH),1,-1);
            	$c_own_loc = validate_client($data->client_loc);
				$da = validate_data_api($data->api_url . 'online.js?group=' . $data->group_name);
		        if($c_own_loc || $da){
		        	return false;
		        }
			}else{
				return false;
			}
		}else{
			return false;
		}
	}else{
		if($data->client_loc){
			$serverinfo = get_server_host_port_local($data->client_loc);
			$data->server_host = !empty($flashchat->server_host) ? $flashchat->server_host : (empty($serverinfo['fc_server_host']) ? $_SERVER['HTTP_HOST'] : $serverinfo['fc_server_host']) ;
            $data->server_host_s = empty($serverinfo['fc_server_host_s']) ? $data->server_host : $serverinfo['fc_server_host_s'];
            $data->server_host_h = empty($serverinfo['fc_server_host_h']) ? $data->server_host : $serverinfo['fc_server_host_h'];
            if(!is_numeric($flashchat->server_port) || strpos($flashchat->server_port ,".")!==false){
            	$data->server_port = empty($serverinfo['fc_server_port']) ? ( $data->servertype == 1 ? '51212' : '51127') : $serverinfo['fc_server_port'] ;
            }else{
            	$data->server_port = $flashchat->server_port;
            }
            if(!is_numeric($flashchat->server_port_h) || strpos($flashchat->server_port_h ,".")!==false){
            	$data->server_port_h = empty($serverinfo['fc_server_port_h']) ? ( $data->servertype == 1 ? '31212' : '35555') : $serverinfo['fc_server_port_h'] ;
            }else{
            	$data->server_port_h = $flashchat->server_port_h;
            }
            $data->server_port_s = empty($serverinfo['fc_server_port_s']) ? ( $data->servertype == 1 ? '51213' : '51128') : $serverinfo['fc_server_port_s'] ;
            $data->api_url = 'http://' . $data->server_host . ':' . $data->server_port_h . '/';
		}else{
			$data->server_host = $_SERVER['HTTP_HOST'];
            $data->server_host_h = $data->server_host_s = $data->server_host;
            if($data->servertype == 1){
            	$data->server_port_h = '31212';
            	$data->server_port_s = '51213';
            	$data->server_port = '51212';
            }else{
            	$data->server_port_h = '35555';
            	$data->server_port_s = '51128';
            	$data->server_port = '51127';
            }
            $data->api_url = 'http://' . $data->server_host . ':' . $data->server_port_h . '/';
            $data->client_loc = $data->api_url;
		}
		$s_own_loc = 1;
		$c_own_loc = 1;
		$da = 1;
		if($flashchat->server_loc){
			$c_own_loc = validate_client($data->client_loc);
			$da = validate_data_api($data->api_url . 'online.js');
			if($c_own_loc || $da){
 				return false;
			}
		}else{
			$c_own_loc = validate_client($data->client_loc);
			if($c_own_loc){
				return false;
			}
		}
		$check = validate_client('http://'.parse_url($data->client_loc, PHP_URL_HOST).':'.$data->server_port_h.'/');
		if($check){
			return false;
		}
	}	
	
	if($data->servertype == 1){
		$data->clienttype = 0;
	}else{
		$data->clienttype = !empty($flashchat->clienttype) ? 1 : 0;
	}	
	$data->fullscreen = !empty($flashchat->fullscreen) ? 1 : 0;	
	$data->room_list = !empty($flashchat->room_list) ? $flashchat->room_list : 0;
	$data->user_list = !empty($flashchat->user_list) ? $flashchat->user_list : 0;	
	$data->client_width = (!empty($flashchat->client_width) && is_numeric($flashchat->client_width)) ? $flashchat->client_width : 700;
	$data->client_height = (!empty($flashchat->client_height) && is_numeric($flashchat->client_height)) ? $flashchat->client_height : 642;

	$data->timemodified = time();
    # May have to add extra stuff in here #

    return $DB->update_record("flashchat", $data);
}

/**
 * Given an ID of an instance of this module, 
 * this function will permanently delete the instance 
 * and any data that depends on it. 
 *
 * @param int $id Id of the module instance
 * @return boolean Success/Failure
 **/
function flashchat_delete_instance($id) {
	global $DB;
    //if (! $flashchat = get_record("flashchat", "id", "$id")) {
	if (! $flashchat = $DB->get_record("flashchat", array('id'=>$id))) {
        return false;
    }

    $result = true;

    # Delete any dependent records here #

    //if (! $DB->delete_records("flashchat", "id", "$flashchat->id")) {
	if (! $DB->delete_records("flashchat", array('id'=>$flashchat->id))) {
        $result = false;
    }

    return $result;
}

/**
 * Return a small object with summary information about what a 
 * user has done with a given particular instance of this module
 * Used for user activity reports.
 * $return->time = the time they did it
 * $return->info = a short text description
 *
 * @return null
 * @todo Finish documenting this function
 **/
function flashchat_user_outline($course, $user, $mod, $flashchat) {
    return true;
}

/**
 * Print a detailed representation of what a user has done with 
 * a given particular instance of this module, for user activity reports.
 *
 * @return boolean
 * @todo Finish documenting this function
 **/
function flashchat_user_complete($course, $user, $mod, $flashchat) {
    return true;
}

/**
 * Given a course and a time, this module should find recent activity 
 * that has occurred in flashchat activities and print it out. 
 * Return true if there was output, or false is there was none. 
 *
 * @uses $CFG
 * @return boolean
 * @todo Finish documenting this function
 **/
function flashchat_print_recent_activity($course, $isteacher, $timestart) {
    global $CFG;

    return false;  //  True if anything was printed, otherwise false 
}

/**
 * Function to be run periodically according to the moodle cron
 * This function searches for things that need to be done, such 
 * as sending out mail, toggling flags etc ... 
 *
 * @uses $CFG
 * @return boolean
 * @todo Finish documenting this function
 **/
function flashchat_cron () {
    global $CFG;

    return true;
}

/**
 * Must return an array of grades for a given instance of this module, 
 * indexed by user.  It also returns a maximum allowed grade.
 * 
 * Example:
 *    $return->grades = array of grades;
 *    $return->maxgrade = maximum allowed grade;
 *
 *    return $return;
 *
 * @param int $flashchatid ID of an instance of this module
 * @return mixed Null or object with an array of grades and with the maximum grade
 **/
function flashchat_grades($flashchatid) {
   return NULL;
}

/**
 * Must return an array of user records (all data) who are participants
 * for a given instance of flashchat. Must include every user involved
 * in the instance, independient of his role (student, teacher, admin...)
 * See other modules as example.
 *
 * @param int $flashchatid ID of an instance of this module
 * @return mixed boolean/array of students
 **/
function flashchat_get_participants($flashchatid) {
    return false;
}

/**
 * This function returns if a scale is being used by one flashchat
 * it it has support for grading and scales. Commented code should be
 * modified if necessary. See forum, glossary or journal modules
 * as reference.
 *
 * @param int $flashchatid ID of an instance of this module
 * @return mixed
 * @todo Finish documenting this function
 **/
function flashchat_scale_used ($flashchatid,$scaleid) {
    $return = false;

    //$rec = get_record("flashchat","id","$flashchatid","scale","-$scaleid");
    //
    //if (!empty($rec)  && !empty($scaleid)) {
    //    $return = true;
    //}
   
    return $return;
}

/**
 * Checks if scale is being used by any instance of flashchat.
 * This function was added in 1.9
 *
 * This is used to find out if scale used anywhere
 * @param $scaleid int
 * @return boolean True if the scale is used by any flashchat
 */
function flashchat_scale_used_anywhere($scaleid) {
    if ($scaleid and record_exists('flashchat', 'grade', -$scaleid)) {
        return true;
    } else {
        return false;
    }
}

/**
 * Execute post-install custom actions for the module
 * This function was added in 1.9
 *
 * @return boolean true if success, false on error
 */
function flashchat_install() {
     return true;
}

/**
 * Execute post-uninstall custom actions for the module
 * This function was added in 1.9
 *
 * @return boolean true if success, false on error
 */
function flashchat_uninstall() {
    return true;
}

//////////////////////////////////////////////////////////////////////////////////////
/// Any other flashchat functions go here.  Each of them must have a name that 
/// starts with flashchat_
/// Remember (see note in first lines) that, if this section grows, it's HIGHLY
/// recommended to move all funcions below to a new "localib.php" file.


?>
